#pragma rtGlobals=1		// Use modern global access method.
#pragma version = 0.75

Macro Create2DFitPanel()
	SetDataFolder root:
	DoWindow/F Fit2DPanel
	if(!V_Flag )
		Create2DFitGlobals()
		MakeFitSetupPanel()
	endif
end

Function/S TempPopupList()
// Make the list of waves for temperature data, allowing "Use Wave Scaling" or a wave with the right number of data points
	String list
	NVAR Ntemps = root:IR2DFitting:gNumTemps
	String theWave, newlist
	
	Variable i=0
	
	newlist = "UseWaveScaling;"
	list =WaveList("*", ";", "DIMS:1")	// Get all single-dimension waves
	
	do
		theWave = StringFromList(i, list)
		if ( strlen(theWave) == 0 )
			break
		endif
		if ( numpnts($theWave) == Ntemps )	// Select only waves that have a number of points = number of temperatures in data
			newlist +=theWave+";"
		endif
		i+=1
	while (i<ItemsInList(list, ";"))
	
	return newlist
End

Function Create2DFitGlobals()
	String dfSav= GetDataFolder(1)                 
	
	NewDataFolder/O/S root:IR2DFitting
	
	Variable/G gNGauss=1
	Variable/G gNTrans
	Variable/G gUseBL=0 
	Variable/G gUseHold=0
	Variable/G gNumTemps      //number of temperatures to fit
	String/G gCoeffWaveName="root:IR2DFitting:coeff"
	String/G gCoeffHoldName="root:IR2DFitting:coeff_hold"
	String/G gSampleDataName
	String/G gTempDataName		// "Temp" is for temperature - I will set it to empty string if using wave scaling

	NewDataFolder/O/S root:IR2DFitting:Results
	SetDataFolder dfSav
end

Proc MakeFitSetupPanel()
	PauseUpdate; Silent 1		// building window...
	String dfSav= GetDataFolder(1)
//	String t_list=""
	
	SetDataFolder root:IR2DFitting

	NewPanel/W=(1040,39,1267,445)/K=1
	DoWindow/C Fit2DPanel
	SetDrawLayer UserBack
	SetDrawEnv rounding= 20,fillpat=0,save
	
	 // the Data Box
	DrawRRect 9,15,217,108
	SetDrawEnv fstyle= 1,textrgb= (0,0,65535),textxjust= 1
	DrawText 31,16,"Data"
	PopupMenu popupSampleData,pos={17,23},size={85,19},title="Data Matrix:  "
	PopupMenu popupSampleData,mode=1,proc=SetDataWaveProc,value= WaveList("*", ";" ,"DIMS:2")   // DIMS:2 selects only 2-dimensional waves
	PopupMenu popupTempData,pos={17,50},size={85,19},title="Temperature:   "
	PopupMenu popupTempData,mode=1, proc=SetTemperatureWaveProc, value=TempPopupList()

	// the Settings Box
	DrawRRect 9,126,217,249 
	SetDrawEnv fstyle= 1,textrgb= (0,0,65535)
	DrawText 15,126,"Settings"
//	DrawLine 9,172,215,172
	SetDrawEnv fsize= 10,textxjust= 2,textyjust= 1
//	DrawText 75,160,"DeltaAbs:"
	SetDrawEnv fname= "Geneva",fsize= 9,textxjust= 2,textyjust= 1
//	DrawText 75,189,"# Gauss terms:"
	SetVariable setNGauss,pos={17,145},size={120,19},title="# Gaussian terms:"
	SetVariable setNGauss,fSize=10,limits={1,8,1},value=gNGauss                               // Number of Gaussian terms to use
	CheckBox checkBL,pos={17,170},size={16,20},value=0, title="Linear baseline at each temperature", proc=SetBaselineOption
	CheckBox checkHold,pos={17, 195},size={16,20},value=0, title="Use Parameter Hold matrix", proc=SetHoldOption

    // the Control Buttons Box
	DrawRRect 7,277,218,400
//	DrawLine 9,145,215,145  // horizontal
	SetDrawEnv fstyle= 1,textrgb= (0,0,65535)
	DrawText 17,275,"Control Buttons"
	Button ParamButton,pos={51,289},size={114,25},title="Parameters",proc=paramProc  // (change to show parameters table)
	Button GraphButton, pos={51,326},size={114,25},title="Update Graph", proc=graphProc     // purpose is to graph the initial parameters to get close before fitting
	Button DoFitButton,pos={51,363},size={114,25},title="Do Fit",proc=fitProc
	
	SetDataFolder dfSav
EndMacro

Function SetDataWaveProc(ctrlName, popNum, popStr) : PopupMenuControl
	String ctrlName
	Variable popNum
	String popStr	
	NVAR Ntemps = root:IR2DFitting:gNumTemps
	SVAR gSampleDataName= root:IR2DFitting:gSampleDataName

	gSampleDataName = popStr
	Ntemps = DimSize($gSampleDataName,1)

End

Function SetTemperatureWaveProc(ctrlName, popNum, popStr) : PopupMenuControl
	String ctrlName
	Variable popNum
	String popStr
	SVAR gTempDataName= root:IR2DFitting:gTempDataName
	NVAR Ntemps = root:IR2DFitting:gNumTemps
	
	if (popNum==1)							// Corresponds to "Use Wave Scaling"
		gTempDataName = "" 				// Empty string if using sample wave scaling
	else  
		gTempDataName = popStr				// Set global variable to temperature data name
	endif
End

Function SetBaselineOption(ctrlName, value) : SetVariableControl
	String ctrlName
	Variable value	
	NVAR gUseBL=root:IR2DFitting:gUseBL

	gUseBL = value
End

Function SetHoldOption(ctrlName, value) : SetVariableControl
	String ctrlName
	Variable value	
	NVAR gUseHold=root:IR2DFitting:gUseHold

	gUseHold = value
End

Function fitProc(ctrlName) : ButtonControl
	String ctrlName
	
	if ( !WaveExists( $"root:IR2DFitting:listGaussParam") )   
		Abort "Please edit the parameters before trying the fit"
	endif
	
	 String func= ""

	NVAR gUseHold=root:IR2DFitting:gUseHold
	SVAR gSampleDataName= root:IR2DFitting:gSampleDataName
 	SVAR gTempDataName= root:IR2DFitting:gTempDataName
 	SVAR gCoeffWaveName= root:IR2DFitting:gCoeffWaveName
	SVAR gCoeffHoldName= root:IR2DFitting:gCoeffHoldName
	NVAR gNumTemps = root:IR2DFitting:gNumTemps

	if (gUseHold)
		Wave hold = $gCoeffHoldName
	endif
	
	ControlInfo popupSampleData
	gSampleDataName= S_value

	String s  //= func

	String dfSav= GetDataFolder(1)
	SetDataFolder root:IR2DFitting
	MuxParameters()     //  coeff and coeff_hold will be made int IR2DFitting folder
	SetDataFolder dfSav

	if (!waveExists(root:IR2DFitting:Results:my_fit) )
		Duplicate $gSampleDataName root:IR2DFitting:Results:my_fit   //  my_fit is in Results
	endif
	
	if (gUseHold)
		String holdstr=""  
		Variable i = 0
		do
			holdstr+=num2str(hold[i])
			i+=1
		while (i < numpnts(hold) )
		s = "FuncFitMD/M=2/H=\""+holdstr+"\" CalcModel "+gCoeffWaveName+" "+gSampleDataName+"/D=root:IR2DFitting:Results:my_fit"    
	else
		s = "FuncFitMD/M=2 CalcModel "+gCoeffWaveName+" "+gSampleDataName+"/D=root:IR2DFitting:Results:my_fit"    
	endif
	if ( exists( gTempDataName) )
		s+="/Y="+gTempDataName
	endif	
	print s
	Execute s
	
	Wave fit_results=$gCoeffWaveName
	DeMuxParameters(fit_results)
	
	DoWindow/K InitParams   			// Kill the initial guesses graph
	DoWindow/K Results        		// Kill any previous Results window
	MakeGraph("Results", root:IR2DFitting:Results:my_fit)
	
	SetDataFolder dfSav
End

Function graphProc(ctrlname):  ButtonControl
	String ctrlName
	String windowName="InitParams"
	
	if ( WaveExists( $"root:IR2DFitting:listGaussParam") )   
		CalculateGuess()
	else
		Abort "Please edit the parameters before graphing initial guesses"
	endif
	
	Wave smooth_curve = $"root:IR2DFitting:init_guess"   			  //  curve based on initial guesses
	DoWindow/F $windowName
	if(!V_flag)                             						 //  if InitParams window doesn't exist
		MakeGraph(windowName, smooth_curve)             // using folder = root:
	endif

End

Macro ShowTemperatureGraph()
	ShowTransitionGraph()
End

Function ShowTransitionGraph()             //  Will need to be changed
	Wave/T listTransResult = $"root:IR2DFitting:Results:listTransResult"
	SVAR gSampleDataName=root:IR2DFitting:gSampleDataName

	Wave datawave = $gSampleDataName
	Variable TT   // Transition temperature
	Variable TS  //  Transition slope
	Variable BK  // Beginning Kelvin value for graph
	Variable EK //  Ending Kelvin value for graph
	BK = DimOffset($gSampleDataName, 1)
	EK = BK+(DimSize($gSampleDataName,1)-1)*DimDelta($gSampleDataName,1)
	
	if (numpnts(listTransResult)==0)
		Abort "The data was fit without temperature transitions - there is no temperature transition graph"
	endif
	
	Variable deltaH, deltaS
	String msg = "Thermodynamic parameters:\r"
	
	Variable i = 0
	Variable cols =DimSize(listTransResult, 0)
	Make/O/N=(256,cols) TransGraph
	SetScale x BK, EK, "", TransGraph
	Display 
	do
		TT=str2num(listTransResult[i][1])
		TS=str2num(listTransResult[i][2])
		Print TT, TS
		TransGraph[][i] = 1/(1+exp(-(1/TT-1/x)*TS))
		AppendToGraph TransGraph[][i]
		deltaH = 8.314*TS/1000
		deltaS = 1000*deltaH/TT
		msg = msg+"Transition Type #"+num2str(i+1)+"\rdeltaH = "+num2str(deltaH)+" kJ/mol\rdeltaS = "+ num2str(deltaS)+ " J/mol/K\r"
		i+=1
	while (i < DimSize(listTransResult, 0) )
	TextBox/C/N=text0/F=0/A=MC msg
	SetAxis left 0,1
	Label left "Fraction of total"
	Label bottom "Temperature (Kelvin)"
End

Function CalculateGuess()    // called with data folder = root:
	NVAR Ntemps = root:IR2DFitting:gNumTemps
	SVAR gCoeffWaveName= root:IR2DFitting:gCoeffWaveName
	SVAR gSampleDataName=root:IR2DFitting:gSampleDataName
 	SVAR gTempDataName= root:IR2DFitting:gTempDataName
	SVAR gCoeffHoldName= root:IR2DFitting:gCoeffHoldName
	NVAR gNumTemps = root:IR2DFitting:gNumTemps
	Wave datawave=$gSampleDataName
	
	MuxParameters()
	Wave current=$gCoeffWaveName   // This needs to be after MuxParameters because that routine creates the coeff wave	
	
	ControlInfo popupSampleData
	gSampleDataName= S_value
	Duplicate/O $gSampleDataName root:IR2DFitting:init_guess

	String s  

	String dfSav= GetDataFolder(1)
	SetDataFolder dfSav

	s = "FuncFitMD/M=2/O/Q CalcModel "+gCoeffWaveName+" "+gSampleDataName+"/D=root:IR2DFitting:init_guess"    
	if ( exists( gTempDataName) )
		s+="/Y="+gTempDataName
	endif	
	print s
	Execute s
	
End

Function MakeGraph(windowName, smooth_curve)     // Procedure for before or after fitting; smooth_curve is the calculated curve from initial guesses or fitting
	String windowName
	Wave smooth_curve
	SVAR gSampleDataName=root:IR2DFitting:gSampleDataName
	NVAR Ntemps = root:IR2DFitting:gNumTemps
	Variable r, g, b, scale
	Wave wave2display = $gSampleDataName
	
	Variable nSkip = Ntemps/5   // put about 5 spectra in the graph
	nSkip = round(nSkip)
	
	ColorTab2Wave Rainbow
	Wave M_colors
	Reverse/DIM=1 M_colors   // This should make the lowest temperatures blue and the highest red rather than vice-versa
	
	Display/K=1/W=(5, 179, 770, 519)/N=$windowName
	Variable i = 0
	Variable nCount = 0
	do
		scale = i*dimSize(M_colors,0)/Ntemps
		r = M_colors[scale][0]
		g = M_colors[scale][1]
		b = M_colors[scale][2]
		AppendToGraph wave2display[][i]   // this append is not working - can't find the data wave
		ModifyGraph/Z rgb[nCount]=( r, g, b )
		ModifyGraph/Z mode[nCount]=3,marker[nCount]=19,msize[nCount]=1.5,mrkThick[nCount]=1.5
		nCount+=1
	// add model spectrum 
		AppendToGraph smooth_curve[][i]    
		ModifyGraph/Z rgb[nCount] = ( r, g, b )  // keep the same color, but one more trace (increment of nCount above)
		nCount+=1
		i+=nSkip
	while ( i <= Ntemps)	
	killWaves M_colors
End

Function paramProc(ctrlName) : ButtonControl
	String ctrlName
	NVAR gNTrans=root:IR2DFitting:gNTrans

	String dfSav= GetDataFolder(1)
	SetDataFolder root:IR2DFitting

	String windowName = "IRMatrixFitting"
	DoWindow/F $windowName
	if(!V_Flag )                                     //If the table doesn't already exist
		NewParametersTable()
	endif
	
	SetDataFolder dfSav
End

Function NewParametersTable()
	NVAR gNGauss=root:IR2DFitting:gNGauss
	NVAR gNTrans=root:IR2DFitting:gNTrans
	SVAR gSampleDataName= root:IR2DFitting:gSampleDataName
	String longSampleDataname = "root:"+gSampleDataName
	
	Variable left = 45
	Variable top = 105
	Variable right = 1105
	Variable bottom = 355
	
	Variable val
	
	gNtrans=gNGauss-1  			// Each Gaussian has a temperature transition to start except first one
	Variable i = 0
	Variable guess = 0               			//  0 = don't guess, 1 = guess initial values
	if ( !WaveExists(listGaussParam) )     // if guesses don't exist then guess initial values
		Make/T/O/N=(gNGauss,13) listGaussParam
		Make/O/N=(gNGauss,13) selGaussParam
		Make/T/O/N=(gNTrans,5) mytemp
		Make/O/N=(gNTrans,5) mySel
	
		for (i=0;i<gNGauss;i+=1)
			listGaussParam[i][12] = num2str(i)
			selGaussParam[i][12] = 2
		endfor  
		guess = 1
	endif
	
// Section on guessing initial values for Gaussian peaks
	if ( guess == 1)  
		Wave datawave = $longSampleDataName
		Variable xmin = DimOffset(datawave,0)
		Variable xmax = xmin+DimSize($longSampleDataName,0)*DimDelta($longSampleDataName,0)
		i = 0
		do
			val = xmin + (i+1)*(xmax-xmin)/(gNGauss+1)       //  Evenly distribute the center positions based on wave scaling
			listGaussParam[i][0] = num2str(val)
			selGaussParam[i][0] = 2
			selGaussParam[i][1] = 0x20                      // Ox20 means "make a checkbox"
			listGaussParam[i][2] = "20"    					  // 20 cm-1 is a good guess
			selGaussParam[i][2] = 2
			selGaussParam[i][3] = 0x20
			val = WaveMax(datawave)/2				
			ListGaussParam[i][4] =  num2str(val) 				// something on the order of the size of the data
			selGaussParam[i][4] = 2
			selGaussParam[i][5] = 0x20
			listGaussParam[i][6] = "0.0"
			selGaussParam[i][6] = 2
			selGaussParam[i][7] = 0x20
			listGaussParam[i][8] = "0.07"						//  Based on Amunson and Kubelka (2007)
			selGaussParam[i][8] = 2
			selGaussParam[i][9] = 0x20
			listGaussParam[i][10] = "0.01" 					  //  So far just a guess - check data 
			selGaussParam[i][10] = 2
			selGaussParam[i][11] = 0x20
			i+=1
		while ( i<gNGauss )
		i = 0
		do
			mytemp[i][1] = num2str(320+10*i )     	    // May have to change this one
			mytemp[i][3] = num2str(20000-400*i)                // Put a transition in the measurable region
			i+=1
		while ( i<=gNtrans )
	endif
	
	NewPanel /N=IRMatrixFitting/K=1/W=(left,top,right,bottom) as "IR Matrix Fitting"
	SetWindow IRMatrixFitting
	ModifyPanel fixedSize=1
	
	DefineGuide splitLeft={FL, 760}
	NewPanel /FG=(FL,FT,splitLeft,FB)/N=PeakParameters/HOST=IRMatrixFitting
	ModifyPanel /W=IRMatrixFitting#PeakParameters frameStyle=1, FrameInset=0
	Make /O/T/N=13 root:IR2DFitting:gaussParamTitles = {"Center", "Hld","Width","Hld","MinAmp","Hld","DeltaAmp","Hld","Center Slope","Hld","Width Slope","Hld","Type"}
	Wave /T gaussParamTitles=root:IR2DFitting:gaussParamTitles
	
	TitleBox transTitle win=IRMatrixFitting#PeakParameters,fsize=12,pos={5,15},frame=0,title="\f01Gaussian Parameters\f00\r\rInitial Guesses:"
	ListBox gaussParamsLB, win=IRMatrixFitting#PeakParameters,titleWave=gaussParamTitles, listWave=listGaussParam,selWave=selGaussParam; DelayUpdate
	ListBox gaussParamsLB, win=IRMatrixFitting#PeakParameters,disable=0, pos={5,85}, size={750,150},fsize=10,mode=6,userColumnResize=0, frame=1
	ListBox gaussParamsLB, win=IRMatrixFitting#PeakParameters,widths={7,2,7,2,7,2,7,2,7,2,7,2,4}, editStyle=1,proc=fPeakParametersBoxProc 
			
	NewPanel /FG=(splitLeft,FT,FR,FB)/N=TransitionParameters/HOST=IRMatrixFitting
	ModifyPanel /W=IRMatrixFitting#TransitionParameters frameStyle=1, FrameInset=0
	
	Make /O/T/N=5 root:IR2DFitting:transParamTitles = {"Type", "Temperature (K)", "Hld", "Slope","Hld"}
	Wave /T transParamTitles=root:IR2DFitting:transParamTitles

	ParamTableTransBox()

	TitleBox transTitle win=IRMatrixFitting#TransitionParameters,fsize=12,pos={5,15},frame=0,title="\f01Temperature Transition Parameters\f00\r\rInitial Guesses:"
	ListBox initParamsLB, win=IRMatrixFitting#TransitionParameters, pos={5,85}, size={290,150}, listWave = mytemp; DelayUpdate
	ListBox initParamsLB, selWave=mySel,disable=0,fsize=10,mode=6,userColumnResize=0,titleWave=transParamTitles,frame=1, editStyle=1
	ListBox initParamsLB, widths={2,4,1,4,1}
	
End

Function fPeakParametersBoxProc(LBstruct) : ListboxControl
	STRUCT WMListboxAction &LBstruct
	NVAR gNTrans = root:IR2DFitting:gNTrans
	SVAR gCoeffHoldName= root:IR2DFitting:gCoeffHoldName
	Variable PperGauss = 6
	Variable val, i
	String tempList=""
	
	Wave/T listTransParam = $"root:IR2DFitting:mytemp"
	Wave selTransParam = $"root:IR2DFitting:mySel"
	
	Wave hold = $gCoeffHoldName
	
	if (LBstruct.eventCode == 13 )   		// Checkbox has changed - change if to switch
		val=(0x10&LBstruct.selWave[LBstruct.row][LBstruct.col])     
		i = LBstruct.row*6 + LBstruct.col/2-1
		if (val == 16 )
			hold[i] = 1		//			Checkbox on
		elseif ( val == 0 )
			hold[i] = 0		//			Checkbox off
		else 	
			Print "Error!"
		endif
	elseif (LBstruct.eventCode == 7 )  										// Finished editing cell
		Make/O/N=20 tempWave
		val = str2num(LBstruct.listWave[LBstruct.row][LBstruct.col] )
		if (numtype(val) == 0 ) 											 // if it is an actual number
			LBstruct.listWave[LBstruct.row][LBstruct.col] =  num2str(val)
		endif
		i=0
		do
			tempList=AddListItem(LBstruct.listWave[i][12], tempList)   // Item 12 in the list is the Transition Type
			i+=1
		while ( i < DimSize(LBstruct.listWave,0) )
		tempList = SortList(tempList)
		val = FindNumItems(tempList, tempWave)
		i = 0
		
		if (val < gNTrans )  												// The number of transitions went down
			Redimension/N=((val),5) listTransParam, selTransParam 
		elseif ( val == gNTrans ) 										 // The transitions haven't changed
			// do nothing
		else  															// The number of transitions went up
			Redimension/N=((val),5) listTransParam, selTransParam 
			i = gNTrans
			do 
				selTransParam[i][0] = 0								// Put some values into the new spots
				listTransParam[i][1] = num2str(320+10*i )
				selTransParam[i][1] = 2
				selTransParam[i][2] = 0x20
				listTransParam[i][3] = num2str(20000-400*i)
				selTransParam[i][3] = 2
				selTransParam[i][4] = 0x20
				i+=1
			while (i < val )
		endif
		do
			listTransParam[i][0] = num2str(tempwave[i])  // Copy the unique list of transition types to the list box
			i+=1
		while (i < numpnts(tempwave) )
		gNTrans = val  
		KillWaves tempWave
	endif
	
	return 0
End

Function FindNumItems(list,returnWave)   // Need number of unique items in the sorted list, except zero (= # of temperature transitions)
	String list
	Wave returnWave
	Variable num=0
	Variable i, length
	String oldval,newval,newlist
	
	list = RemoveFromList ("0", list)	// Zero indicates no temperature transition - get rid of them
	length= ItemsInList(list, ";")
	if ( length ==0 )                               //  All entries are zero
		Redimension/N=0 returnWave
		return 0
	endif
	oldval = StringFromList(0, list)
	newlist =oldval+";"
	if ( length == 1 )					// With only one entry, the list must have one unique value
		Redimension/N=1 returnWave
		returnWave[0] = str2num(oldval)
		return 1
	endif
	
	i=1
	num+=1
	do
		newval = StringFromList(i,list)
		i+=1
		if ( (FindListItem(newval,newlist)==-1)  )  // If newval is not already in the list
			newlist = AddListItem(newval, newlist)
			num+=1
		endif
		newval = oldval
	while (i<length)
	newlist = SortList(newlist)
	Redimension/N=(num) returnWave
	i = 0
	do
		returnWave[i]= str2num(StringFromList(i,newlist))
		i+=1
	while (i < num )
	return num
End

Function ParamTableTransBox()
	NVAR gNGauss=root:IR2DFitting:gNGauss
	NVAR gNTrans=root:IR2DFitting:gNTrans

	Wave/T listTransParam = $"root:IR2DFitting:mytemp"
	Wave selTransParam = $"root:IR2DFitting:mySel"
	
	Variable i=0	
	do 
		listTransParam[i][0]=num2str(i+1)
		selTransParam[i][0] = 0
		selTransParam[i][1] = 2
		selTransParam[i][2] = 0x20
		selTransParam[i][3] = 2
		selTransParam[i][4] = 0x20
		i+=1
	while (i<gNTrans)
	ListBox initParamsLB, win=IRMatrixFitting#TransitionParameters, listWave = listTransParam, selWave = selTransParam

End

Function CalcModel(w, wavenum, Kelvin) : FitFunc 
// w contains the parameters to calculate the wave value; the function returns one absorbance value
// wavenum is the wavenumber value at which to calculate absorbance
// Kelvin is the temperature at which to calculate absorbance
	Wave w 
	Variable wavenum, Kelvin
	
	Variable pnum, mysum=0
	Variable  j, accum              // Accumulate the Gaussians
	Variable slope, yintc			// These are used for linear baseline parameters
	Variable PperGauss = 6           // For current set-up, there are 6 parameters per gaussian term
	NVAR gNGauss=root:IR2DFitting:gNGauss
	NVAR useBL = root:IR2DFitting:gUseBL
	SVAR gSampleDataName= root:IR2DFitting:gSampleDataName
	Variable NGaussParms
	NGaussParms=gNGauss*PperGauss  // For when Nterms changes
	Wave/T listGaussParam = $"root:IR2DFitting:listGaussParam"
	Variable RefT = 298
	Variable linParm
	Variable c_temp, w_temp    //temporary variables for center position, width because they change with temperature
	Variable temp_slope, temp_trans   // temporary variables to store the temperature transition parameters
	Variable i=0                        // Counter to go through Nterms gaussian terms
		
	
	do
		pnum = PperGauss*i
		c_temp=w[pnum]+w[pnum+4]*(Kelvin-RefT)           // c_temp is necessary to find the center position of a gaussian as Kelvin varies
		w_temp=w[pnum+1]+w[pnum+5]*(Kelvin-RefT)     // w_temp is necesary to find the width of a gaussian as Kelvin varies
		accum = exp(-(wavenum-c_temp)^2/w_temp^2)
		if ( str2num(listGaussParam[i][12] ) != 0 )               // There is a temperature transition
			j = NGaussParms + (str2num(listGaussParam[i][12] )-1)*2   //  Finds the temperature transition number
			temp_trans = w[j]
			temp_slope = w[j+1]
			accum = accum*((w[pnum+3])/(1+exp( (1/Kelvin-1/temp_trans) *temp_slope))  +(w[pnum+2])) 
		else
			accum = accum*w[pnum+2]           // 
		endif
		mysum+= accum  // Add in the ith gaussian curve
		i+=1
	while (i<gNGauss)
	
	if (useBL==1)   // then add the baseline
		Variable center
		Wave datawave = $gSampleDataName
		center = DimOffset(datawave, 0)           // It's not really the center yet
		linParm = Temp2Params(Kelvin)        // Find the parameter number 
		slope=w[linParm]    
		yintc=w[linParm+1] 
		mysum+=yintc+slope*(wavenum-center)  // Later change the 1600 to center of range
	endif
	return mysum
End

Function Temp2Params(Temp)     //  This function calculates which spectrum corresponds to a certain temperature and returns the 
	Variable Temp                          //       first parameter number for a linear baseline at that temperature
	NVAR gNGauss=root:IR2DFitting:gNGauss
	NVAR gNTrans=root:IR2DFitting:gNTrans
	SVAR gSampleDataName=root:IR2DFitting:gSampleDataName
	Variable ParamNum
	Variable PperGauss=6
	
	ParamNum=(Temp - DimOffset($gSampleDataName, 1))/DimDelta($gSampleDataName,1)
	ParamNum=round(ParamNum)*2
	ParamNum+=gNGauss*PperGauss + gNTrans*2  // Two parameters per temperature transition
	return ParamNum
End

Function DeMuxParameters(coeff)
	Wave coeff
	
	NVAR gNGauss=root:IR2DFitting:gNGauss
	NVAR gNTrans=root:IR2DFitting:gNTrans
	NVAR useBL = root:IR2DFitting:gUseBL
	NVAR Ntemps = root:IR2DFitting:gNumTemps
	
	Wave/T listGaussParam = $"root:IR2DFitting:listGaussParam"
//	Wave selGaussParam = $"root:IR2DFitting:selGaussParam"
	Wave/T listTransParam = $"root:IR2DFitting:mytemp"
//	Wave selTransParam = $"root:IR2DFitting:mySel"
	
	String dfSav= GetDataFolder(1)	
	SetDataFolder root:IR2DFitting:Results
	
	Make/T/O/N=(gNGauss,7) listGaussResult
	Make/T/O/N=(gNTrans,3) listTransResult
	
	Variable i = 0
	Variable pnum
	Variable PperGauss=6
	
	do
		pnum = i*PperGauss
		                                                       									// Following are the seven parameters for each Gaussian Term:
		listGaussResult[i][0] = num2str( coeff[pnum] )          			       // Center position (quoted at reference temperature of 298 K)
		listGaussResult[i][1] = num2str( coeff[pnum+1]*1.665109 )		// Width as FWHM (quoted at reference temperature of 298 K)
		listGaussResult[i][2] = num2str( coeff[pnum+2] )     			       // Maximum height (absorbance units) is equal to area/width
		listGaussResult[i][3] = num2str( coeff[pnum+3] )				       //  Minimum height (absorbance units) is equal to area/width (only applicable with a transition)
		listGaussResult[i][4] = num2str( coeff[pnum+4] )  		        	// Slope of center position (wavenumbers per degree)
		listGaussResult[i][5] = num2str( coeff[pnum+5] ) 			       	// Slope of width (wavenumbers per degree)
		listGaussResult[i][6] = listGaussParam[i][12]
		i+=1
	while(i<gNGauss)

	if (gNtrans > 0 )
		i = 1
		do
			pnum=gNGauss*PperGauss + 2*(i-1) 
			listTransResult[i-1][0] = listTransParam[i-1][0]
			listTransResult[i-1][1] = num2str( coeff[pnum] )
			listTransResult[i-1][2] = num2str( coeff[pnum+1] )
			i+=1
		while ( i<=gNtrans )
	endif

	MakeResultsList(listGaussResult,listTransResult)
	SetDataFolder dfSav
End

Function MakeResultsList(listGaussResult, listTransResult)
	Wave listGaussResult, listTransResult
	
	String listWindowName = "IRMatrixResult"
	DoWindow/F $listWindowName
	if(!V_flag)                              //  if InitParams window doesn't exist
		NewPanel /N=$listWindowName/K=1/W=(5,13,755,205) as "Matrix Fitting Result"
	endif
	SetWindow $listWindowName
	ModifyPanel fixedSize=1
	
	DefineGuide splitLeft={FL, 470}
	NewPanel /FG=(FL,FT,splitLeft,FB)/N=PeakParameters/HOST=IRMatrixResult
	ModifyPanel /W=IRMatrixResult#PeakParameters frameStyle=1, FrameInset=0
	Make /O/T/N=7 root:IR2DFitting:gaussTitles = {"Center","Width","MinAmp","DeltaAmp","Center Slope","Width Slope","Type"}
	Wave /T gaussTitles=root:IR2DFitting:gaussTitles
	
	TitleBox transTitle win=IRMatrixResult#PeakParameters,fsize=12,pos={7,15},frame=0,title="\f01Gaussian Parameters\f00\rFitting Results:"
	ListBox gaussParamsLB, win=IRMatrixResult#PeakParameters,titleWave=gaussTitles
	ListBox gaussParamsLB, win=IRMatrixResult#PeakParameters, listWave=listGaussResult//,selWave=selGaussParam; DelayUpdate
	ListBox gaussParamsLB, win=IRMatrixResult#PeakParameters,disable=0, pos={5,60}, size={460,120},fsize=10,mode=6,userColumnResize=0, frame=1
	ListBox gaussParamsLB, win=IRMatrixResult#PeakParameters,widths={6,6,6,6,7,7,4}
			
	NewPanel /FG=(splitLeft,FT,FR,FB)/N=TransitionParameters/HOST=IRMatrixResult
	ModifyPanel /W=IRMatrixResult#TransitionParameters frameStyle=1, FrameInset=0
	
	Make /O/T/N=5 root:IR2DFitting:transTitles = {"Type", "Temp. ", "Slope"}
	Wave /T transTitles=root:IR2DFitting:transTitles

	ParamTableTransBox()

	TitleBox transTitle win=IRMatrixResult#TransitionParameters,fsize=12,pos={7,15},frame=0,title="\f01Temperature Transitions\f00\rFitting Results:"
	ListBox initParamsLB, win=IRMatrixResult#TransitionParameters,titleWave=transTitles
	ListBox initParamsLB, win=IRMatrixResult#TransitionParameters, listWave = listTransResult //selWave=mySel
	ListBox initParamsLB, win=IRMatrixResult#TransitionParameters,disable=0, pos={5,60}, size={270,120},fsize=10,mode=6,userColumnResize=0, frame=1
	ListBox initParamsLB, win=IRMatrixResult#TransitionParameters,widths={2,4,4}
	
End

Function MuxParameters()

	Wave/T listGaussParam = $"root:IR2DFitting:listGaussParam"
	Wave/T listTransParam = $"root:IR2DFitting:mytemp"
	
	NVAR Ntemps = root:IR2DFitting:gNumTemps
	NVAR gNGauss=root:IR2DFitting:gNGauss
	NVAR gNTrans=root:IR2DFitting:gNTrans
	NVAR useBL = root:IR2DFitting:gUseBL
	SVAR gSampleDataName=root:IR2DFitting:gSampleDataName
	String longSampleName="root:"+gSampleDataName
	Variable npts_x
	npts_x = DimSize($longSampleName,0)

	Variable i=0
	Variable pnum
	Variable PperGauss=6            // currently there are 6 parameters for each Gaussian term
	
	Variable Nparams
	Nparams = PperGauss*gNGauss + gNtrans*2    // This excludes any baseline parameters

	Wave datawave=$longSampleName
	String dfSav= GetDataFolder(1)
	SetDataFolder root:IR2DFitting
	
	Make/O/D/N=(Nparams) coeff
	coeff = 0
	if ( !WaveExists(root:IR2DFitting:coeff_hold) )
		Make/D/N=(Nparams) coeff_hold
		coeff_hold = 0
	else
		Redimension/N=(Nparams) coeff_hold
	endif
	
	i=0
	do
		pnum=i*PperGauss
		
		coeff[pnum] = str2num(listGaussParam[i][0])						// Center parameter  (odd numbers are "hold" key)
		coeff[pnum+1] = str2num(listGaussParam[i][2])/1.665109		//  Width - the factor of 1.66 changes sigma to FWHM
		coeff[pnum+2] = str2num(listGaussParam[i][4])   					//  Minimum amplitude
		coeff[pnum+3] = str2num(listGaussParam[i][6])     				//  Amplitude change
		coeff[pnum+4] = str2num(listGaussParam[i][8])  					// center slope
		coeff[pnum+5] = str2num(listGaussParam[i][10])  				//  Width slope

		if ( str2num(listGaussParam[i][12]) == 0 )   // There is no temperature transition
			coeff_hold[pnum+3] = 1				    //  so the amplitude change cannot be varied (it duplicates minimum amplitude)
		endif
		i+=1
	while (i<gNGauss)
	i = 1
	if (gNtrans > 0 )
		do
			pnum=gNGauss*PperGauss + 2*(i-1)
			coeff[pnum] = str2num(listTransParam[i][1])
			coeff[pnum+1] = str2num(listTransParam[i][3])
			i+=1
		while ( i<=gNtrans )
	endif
	
	if (useBL==1)
		Nparams+=Ntemps*2
		Redimension/N=(Nparams) coeff   // assume that the baseline parameters always vary, don't change coeff_hold
		i=0
		Nparams =  PperGauss*gNGauss + gNtrans*2    // This is the number of Gaussian + transition parameters
		do
			coeff[Nparams+i*2] = (datawave[npts_x-1][i]-datawave[0][i])/npts_x    	// For the slope
			coeff[Nparams +i*2+1] = datawave[0][i]                                        			// For the y-intercept
			i+=1
		while (i<Ntemps)
	endif	
	SetDataFolder dfSav

End

Function MyWaveStats(inputWave, outputWave)
	WAVE inputWave
	WAVE outputWave

	WaveStats/Q inputWave

	outputWave[0] = V_npnts
	outputWave[1] = V_avg
	outputWave[2] = V_sdev
End

